// -*- C++ -*-
/*!
 * @file  RTC_E2_Crawler.h
 * @brief E2クローラ動作用RTC
 * @date  $Date$
 *
 * @author AIZUK
 * kohno@aizuk.jp
 *
 * $Id$
 */

#ifndef RTC_E2_CRAWLER_H
#define RTC_E2_CRAWLER_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
void Log_init();
/*!
 * @class RTC_E2_Crawler
 * @brief E2クローラ動作用RTC
 *
 * E2クローラロボット動作用RTC
 * 目標速度等を受けて、
 * モータ制御用マイコンに指令を送る
 *
 * 入力
 * 　　Pose3D：他のセンサ等で推定した機体位置姿勢
 * 　 Velocity_In：機体目標速度
 * Sub_Angle：サブクローラ目標角度
 * Mode：機体制御モード
 * MOT_Main：メインモータのPWM値直接入力
 * MOT_Sub：サブモータのPWM値直接入力
 * 出力
 * 　　Odometry：オドメトリで算出した機体位置姿勢
 * 　 Velocity_Out：オドメトリで算出した機体速度
 *
 */
class RTC_E2_Crawler
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  RTC_E2_Crawler(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~RTC_E2_Crawler();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * SH1(メインモータ)通信速度
   * - Name:  BAUDRATE1
   * - DefaultValue: B57600
   */
  std::string m_BAUDRATE1;
  /*!
   * SH2(サブモータ)通信速度
   * - Name:  BAUDRATE2
   * - DefaultValue: B57600
   */
  std::string m_BAUDRATE2;
  /*!
   * SH3(サブモータ)通信速度
   * - Name:  BAUDRATE3
   * - DefaultValue: B57600
   */
  std::string m_BAUDRATE3;
  /*!
   * SH1(メインモータ)　Port名
   * - Name:  SH1_PORT
   * - DefaultValue: /dev/ttyUSB0
   */
  std::string m_SH1_PORT;
  /*!
   * SH2(サブモータ)　Port名
   * - Name:  SH2_PORT
   * - DefaultValue: /dev/ttyUSB1
   */
  std::string m_SH2_PORT;
  /*!
   * SH3(サブモータ)　Port名
   * - Name:  SH3_PORT
   * - DefaultValue: /dev/ttyUSB2
   */
  std::string m_SH3_PORT;
  /*!
   * pid　ゲイン　右メインモータ pゲイン
   * - Name:  kp_R
   * - DefaultValue: 2.5
   */
  double m_kp_R;
  /*!
   * pid　ゲイン　右メインモータ iゲイン
   * - Name:  ki_R
   * - DefaultValue: 1.9
   */
  double m_ki_R;
  /*!
   * pid　ゲイン　右メインモータ dゲイン
   * - Name:  kd_R
   * - DefaultValue: 2
   */
  double m_kd_R;
  /*!
   * pid　ゲイン　左メインモータ pゲイン
   * - Name:  kp_L
   * - DefaultValue: 2.5
   */
  double m_kp_L;
  /*!
   * pid　ゲイン　左メインモータ iゲイン
   * - Name:  ki_L
   * - DefaultValue: 1.9
   */
  double m_ki_L;
  /*!
   * pid　ゲイン　左メインモータ ｄゲイン
   * - Name:  kd_L
   * - DefaultValue: 2
   */
  double m_kd_L;
  /*!
   * 車輪半径
   * - Name:  Rw
   * - DefaultValue: 0.07681
   * - Unit: [m]
   */
  double m_Rw;
  /*!
   * 車輪間距離
   * - Name:  T
   * - DefaultValue: 0.3288
   * - Unit: [m]
   */
  double m_T;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::TimedPose3D m_Pose3D;
  /*!
   * 外部センサ等で推定したロボット位置姿勢入力
   */
  InPort<RTC::TimedPose3D> m_Pose3DIn;
  RTC::TimedVelocity2D m_Velocity_In;
  /*!
   * ロボット目標速度
   */
  InPort<RTC::TimedVelocity2D> m_Velocity_InIn;
  RTC::TimedDoubleSeq m_Sub_Angle;
  /*!
   * サブクローラ目標角度
   */
  InPort<RTC::TimedDoubleSeq> m_Sub_AngleIn;
  RTC::TimedLongSeq m_Mode;
  /*!
   * 機体制御モード
   */
  InPort<RTC::TimedLongSeq> m_ModeIn;
  RTC::TimedLongSeq m_MOT_Main;
  /*!
   * メインモータ（メインクローラ）PWM値を直接設定
   */
  InPort<RTC::TimedLongSeq> m_MOT_MainIn;
  RTC::TimedLongSeq m_MOT_Sub;
  /*!
   * サブモータ（サブクローラ）PWM値を直接設定
   */
  InPort<RTC::TimedLongSeq> m_MOT_SubIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  RTC::TimedPose3D m_Odometry;
  /*!
   * オドメトリにより算出した機体位置姿勢
   */
  OutPort<RTC::TimedPose3D> m_OdometryOut;
  RTC::TimedVelocity2D m_Velocity_Out;
  /*!
   * オドメトリにより算出した機体速度
   */
  OutPort<RTC::TimedVelocity2D> m_Velocity_OutOut;
  RTC::TimedDoubleSeq m_Load;
  /*!
   * 各モータ負荷
   * - Unit: [A]
   */
  OutPort<RTC::TimedDoubleSeq> m_LoadOut;
  
  
   RTC::TimedDoubleSeq m_Sub_Angle_Out;
  /*!
   * サブクローラ角度
   */
  OutPort<RTC::TimedDoubleSeq> m_Sub_Angle_OutOut;
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>
 virtual RTC::ReturnCode_t Connect_serial(int sh_num);

 virtual RTC::ReturnCode_t  Status();
 virtual RTC::ReturnCode_t  Logger();
 virtual RTC::ReturnCode_t  Sub_ANG(int id , double DEG_T);
 private:
   
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

};


extern "C"
{
  DLL_EXPORT void RTC_E2_CrawlerInit(RTC::Manager* manager);
};

#endif // RTC_E2_CRAWLER_H
